@@ -17,11 +17,10 @@ import os
 
 from bzt import TaurusConfigError, TaurusInternalException
 from bzt.modules import SubprocessedExecutor
-from bzt.engine import HavingInstallableTools
 from bzt.utils import get_full_path, is_windows, RequiredTool, RESOURCES_DIR, CALL_PROBLEMS
 
 
-class CSharpExecutor(SubprocessedExecutor, HavingInstallableTools):
+class CSharpExecutor(SubprocessedExecutor):
     def __init__(self):
         super(CSharpExecutor, self).__init__()
         self.runner_dir = os.path.join(RESOURCES_DIR, "dotnet", "DotnetTestRunner")
@@ -40,6 +39,7 @@ class CSharpExecutor(SubprocessedExecutor, HavingInstallableTools):
 
     def prepare(self):
         super(CSharpExecutor, self).prepare()
+        self.env.set({"CopyRetryCount": 100})
         self.script = get_full_path(self.get_script_path())
         if not self.script:
             raise TaurusConfigError("Script not passed to runner %s" % self)
