@@ -8,8 +8,8 @@ from bzt.engine import ScenarioExecutor
 from bzt.modules.ab import TSVDataReader
 from bzt.modules.aggregator import AggregatorListener, ConsolidatingAggregator, DataPoint
 from bzt.modules.gatling import DataLogReader as GatlingLogReader
-from bzt.modules.grinder import DataLogReader as GrinderLogReader
 from bzt.modules.jmeter import JTLReader, XMLJTLReader
+from bzt.modules.vegeta import VegetaLogReader
 from bzt.utils import dehumanize_time
 
 
@@ -21,7 +21,6 @@ class ExternalResultsLoader(ScenarioExecutor, AggregatorListener):
     PBENCH_FORMAT = re.compile("^[0-9]+\.[0-9]{3}\t[^\t]*\t([0-9]+\t){9}[0-9]+$")
 
     def __init__(self):
-        # TODO: document this executor
         super(ExternalResultsLoader, self).__init__()
         self._result_timeout = 1
         self._file_exists_wait = 1
@@ -100,8 +99,6 @@ class ExternalResultsLoader(ScenarioExecutor, AggregatorListener):
         with open(self.data_file) as fhd:
             header = fhd.readline(2048).strip()  # just header chunk of file
 
-        # TODO: detect CSV dialect for JTLs
-
         if header.startswith(self.AB_HEADER):
             reader = TSVDataReader(self.data_file, self.log)
             reader.url_label = "N/A"
@@ -112,8 +109,9 @@ class ExternalResultsLoader(ScenarioExecutor, AggregatorListener):
             return GatlingLogReader(self.data_file, self.log, None)
         elif "timestamp" in header.lower() and "elapsed" in header.lower():
             return JTLReader(self.data_file, self.log, self.errors_file)
-        elif "worker process" in header.lower() and header.startswith("worker."):
-            return GrinderLogReader(self.data_file, self.log)
+        elif re.match("^[0-9]{19},", header):
+            # Vegeta CSV does not have a header, every line starts with a timestamp in nanoseconds
+            return VegetaLogReader(self.data_file, self.log)
         else:
             self.log.info("Header line was: %s", header)
             raise TaurusInternalException("Unable to detect results format for: %s" % self.data_file)
@@ -132,7 +130,6 @@ class ExternalResultsLoader(ScenarioExecutor, AggregatorListener):
         ts_not_changed = self._last_datapoint_ts == self._prev_datapoint_ts
         no_new_results = time.time() - self._last_update_ts > self._result_timeout
         has_read_some = self._last_datapoint_ts > 0 or bool(self.reader and self.reader.buffer)
-        # self.log.info("%s %s %s", self._last_datapoint_ts, self._prev_datapoint_ts, self._last_update_ts)
         if has_read_some and ts_not_changed and no_new_results:
             return True
         else:
