@@ -21,6 +21,7 @@ import csv
 import fnmatch
 import os
 import re
+import shutil
 import socket
 import tempfile
 import time
@@ -194,10 +195,17 @@ class JMeterExecutor(ScenarioExecutor):
         test_plan = jmx.get(selector)[0]
         ver = test_plan.get('jmeter')
         if isinstance(ver, str):
-            index = ver.find(" ")
-            if index != -1:
-                return ver[:index]
-
+            space_index = ver.find(" ")
+            # return original value if it doesn't contain the space
+            # ver = "4.0" -> return "4.0" (NOT JMeter.VERSION)
+            # ver = "2.13 r1665067" -> return "2.13"
+            if space_index != -1:
+                return ver[:space_index]
+            dash_index = ver.find("-")
+            if dash_index != -1:
+                # not supported version like: 2.12-SNAPSHOT.20150128
+                return JMeter.VERSION
+            return ver
         return JMeter.VERSION
 
     def prepare(self):
@@ -217,8 +225,23 @@ class JMeterExecutor(ScenarioExecutor):
         is_jmx_generated = False
 
         self.original_jmx = self.get_script_path()
-        if self.settings.get("version", JMeter.VERSION, force_set=True) == "auto":
-            self.settings["version"] = self._get_tool_version(self.original_jmx)
+        self.log.debug("Getting Jmeter version.")
+        if os.getenv('TAURUS_JMETER_DISABLE_AUTO_DETECT', 'False') == 'True':
+            self.log.debug("Autodetect version is disabled.")
+            # autodetection NOT active -> use STABLE if auto,
+            # xyz if STABLE|LATEST
+            # STABLE otherwise
+            config_version = self.settings.get("version", JMeter.VERSION, force_set=True)
+            if config_version in [JMeter.VERSION, JMeter.VERSION_LATEST]:
+                self.settings["version"] = config_version
+            else:
+                self.settings["version"] = JMeter.VERSION
+        else:
+            # autodetection IS active (original default behavior)
+            self.log.debug("Autodetect version is enabled")
+            if self.settings.get("version", JMeter.VERSION, force_set=True) == "auto":
+                self.settings["version"] = self._get_tool_version(self.original_jmx)
+        self.log.debug("JMeter version: %s", self.settings["version"])
 
         if not self.original_jmx:
             if self.get_scenario().get("requests"):
@@ -861,7 +884,7 @@ class JTLReader(ResultsReader):
                 trname = row["Hostname"] + row["threadName"][:row["threadName"].rfind('-')]
             else:
                 concur = int(row["allThreads"])
-                trname = row["threadName"] if self._redundant_aggregation else ""
+                trname = ""
 
             rtm = int(row["elapsed"]) / 1000.0
             ltc = int(row["Latency"]) / 1000.0
@@ -1376,6 +1399,7 @@ class JMeter(RequiredTool):
     COMMAND_RUNNER_VERSION = "2.2"
     COMMAND_RUNNER_LINK = 'https://search.maven.org/remotecontent?filepath=kg/apc/cmdrunner/{version}/cmdrunner-{version}.jar'
     VERSION = "5.4.3"
+    VERSION_LATEST = "5.5"
 
     def __init__(self, config=None, props=None, **kwargs):
         settings = config or BetterDict()
@@ -1549,41 +1573,61 @@ class JMeter(RequiredTool):
 
     def _get_jar_fixes(self, lib_dir):
         direct_install_tools = []
-        if self.fix_jars:
-            # these jars should be replaced with newer version in order to fix some vulnerabilities
 
-            # component name and download link in https://repo1.maven.org/maven2/
-            affected_components = {
-                # Needs to be <1.4.18 for now https://stackoverflow.com/questions/30812293/com-thoughtworks-xstream-security-forbiddenclassexception
-                "xstream": "com/thoughtworks/xstream/xstream/1.4.17/xstream-1.4.17.jar",
-                "jackson-annotations": "com/fasterxml/jackson/core/jackson-annotations/2.13.3/jackson-annotations-2.13.3.jar",
-                "jackson-core": "com/fasterxml/jackson/core/jackson-core/2.13.3/jackson-core-2.13.3.jar",
-                "jackson-databind": "com/fasterxml/jackson/core/jackson-databind/2.13.3/jackson-databind-2.13.3.jar",
-                "netty-codec": "io/netty/netty-codec/4.1.77.Final/netty-codec-4.1.77.Final.jar",
-                "jetty-io": "org/eclipse/jetty/jetty-io/9.4.46.v20220331/jetty-io-9.4.46.v20220331.jar",
-                "snakeyaml": "org/yaml/snakeyaml/1.30/snakeyaml-1.30.jar",
-                "json-smart": "net/minidev/json-smart/2.4.8/json-smart-2.4.8.jar",
-                "xmlgraphics-commons": "org/apache/xmlgraphics/xmlgraphics-commons/2.7/xmlgraphics-commons-2.7.jar"}
+        if not self.fix_jars or LooseVersion(self.version) < LooseVersion('5.0.0'):
+            return direct_install_tools
 
-            if LooseVersion(self.version) <= LooseVersion('5.4.2'):  # log4j must be fixed till jmeter 5.4.2
-                affected_names = ["log4j-core", "log4j-api", "log4j-slf4j-impl", "log4j-1.2-api"]
-                fixed_version = '2.17.2'
-                maven_link = "org/apache/logging/log4j/{name}/{ver}/{name}-{ver}.jar"
+        # these jars should be replaced with newer version in order to fix some vulnerabilities
+        # component name and download link in https://repo1.maven.org/maven2/
+        affected_components = {
+            # Needs to be <1.4.18 for old Jmeters https://stackoverflow.com/questions/30812293/com-thoughtworks-xstream-security-forbiddenclassexception
+            "xstream": "com/thoughtworks/xstream/xstream/1.4.19/xstream-1.4.19.jar",
+            "jackson-annotations": "com/fasterxml/jackson/core/jackson-annotations/2.13.4/jackson-annotations-2.13.4.jar",
+            "jackson-core": "com/fasterxml/jackson/core/jackson-core/2.13.4/jackson-core-2.13.4.jar",
+            "jackson-databind": "com/fasterxml/jackson/core/jackson-databind/2.13.4.2/jackson-databind-2.13.4.2.jar",
+            "json-smart": "net/minidev/json-smart/2.4.8/json-smart-2.4.8.jar",
+            "jsoup": "org/jsoup/jsoup/1.15.3/jsoup-1.15.3.jar",
+            "snakeyaml": "org/yaml/snakeyaml/1.33/snakeyaml-1.33.jar",
+            "okhttp": "com/squareup/okhttp3/okhttp/4.10.0/okhttp-4.10.0.jar",
+            "commons-text": "org/apache/commons/commons-text/1.10.0/commons-text-1.10.0.jar",
+            "xmlgraphics-commons": "org/apache/xmlgraphics/xmlgraphics-commons/2.7/xmlgraphics-commons-2.7.jar"}
 
-                for name in affected_names:
-                    affected_components[name] = maven_link.format(name=name, ver=fixed_version)
+        if LooseVersion(self.version) <= LooseVersion('5.4.2'):  # log4j must be fixed till jmeter 5.4.2
+            affected_names = ["log4j-core", "log4j-api", "log4j-slf4j-impl", "log4j-1.2-api"]
+            fixed_version = '2.19.0'
+            maven_link = "org/apache/logging/log4j/{name}/{ver}/{name}-{ver}.jar"
 
-            jar_files = [_file for _file in os.listdir(lib_dir) if _file.endswith(".jar")]
-            for jar_file in jar_files:
-                for comp_name in affected_components:
-                    if jar_file.startswith(comp_name):
-                        download_link = "https://repo1.maven.org/maven2/" + affected_components[comp_name]
-                        target_path = os.path.join(lib_dir, jar_file)
-                        direct_install_tools.append([download_link, target_path])
-                        break
+            for name in affected_names:
+                affected_components[name] = maven_link.format(name=name, ver=fixed_version)
+
+        jar_files = [_file for _file in os.listdir(lib_dir) if _file.endswith(".jar")]
+        for jar_file in jar_files:
+            for comp_name in affected_components:
+                if jar_file.startswith(comp_name):
+                    download_link = "https://repo1.maven.org/maven2/" + affected_components[comp_name]
+                    target_path = os.path.join(lib_dir, jar_file)
+                    direct_install_tools.append([download_link, target_path])
+                    break
 
         return direct_install_tools
 
+    def _fix_jquery_in_jmeter(self, jmeter_dir):
+        if not self.fix_jars or LooseVersion(self.version) < LooseVersion('5.0.0'):
+            return
+
+        # Fix CVE-2016-10707 in jquery
+        jquery_src_dir = os.path.join(jmeter_dir, "jquery-dist-3.6.0")
+        jquery_target_dir = os.path.join(jmeter_dir,
+            "bin/report-template/sbadmin2-1.0.7/bower_components/jquery")
+        jquery_tar = os.path.join(jmeter_dir, "jquery-dist-3.6.0.tar.gz")
+        self.__download_additions([["https://github.com/jquery/jquery-dist/archive/3.6.0.tar.gz",
+                                    jquery_tar]])
+        shutil.unpack_archive(jquery_tar, jmeter_dir)
+        shutil.rmtree(jquery_target_dir, ignore_errors=True)
+        os.makedirs(jquery_target_dir)
+        shutil.move(jquery_src_dir, jquery_target_dir)
+        os.remove(jquery_tar)
+
     def install(self):
         jmeter_dir = get_full_path(self.tool_path, step_up=2)
         lib_dir = os.path.join(jmeter_dir, 'lib')
@@ -1595,14 +1639,14 @@ class JMeter(RequiredTool):
 
         self.__install_jmeter(jmeter_dir)
 
-        direct_install_tools = self._get_jar_fixes(lib_dir)  # format: download link, local destination path
-        direct_install_tools.extend([
-            [self.plugins_manager_link, pm_installer_path],
-            [self.command_runner_link, command_runner_path]])
-
+        direct_install_tools = [[self.plugins_manager_link, pm_installer_path],
+                                [self.command_runner_link, command_runner_path]]
         self.__download_additions(direct_install_tools)
         self.__install_plugins_manager(pm_installer_path)
         self.__install_plugins()
+        # Apply JAR vulnerability fixes
+        self.__download_additions(self._get_jar_fixes(lib_dir))
+        self._fix_jquery_in_jmeter(jmeter_dir)
 
         cleaner = JarCleaner(self.log)
         cleaner.clean(lib_dir)
