@@ -62,7 +62,6 @@ class HTTPRequest(Request):
         if not has_variable_pattern(self.method):
             self.method = self.method.upper()
 
-        # TODO: add method to join dicts/lists from scenario/request level?
         self.headers = self.config.get("headers", {})
 
         self.keepalive = self.config.get('keepalive', None)
@@ -86,7 +85,6 @@ class HTTPRequest(Request):
         return headers.get(name.lower(), None)
 
     def _get_body(self, pure_body_file=False):
-        # todo: if self.method not in ("PUT", "POST")?
         body = self.config.get('body', None)
         body_file = self.config.get('body-file')
         if body_file:
@@ -171,6 +169,30 @@ class OnceBlock(Request):
         return "OnceBlock(requests=%s)" % requests
 
 
+class SetUpBlock(Request):
+    NAME = "setup"
+
+    def __init__(self, requests, config):
+        super().__init__(config)
+        self.requests = requests
+
+    def __repr__(self):
+        requests = [repr(req) for req in self.requests]
+        return "SetUpBlock(requests=%s)" % requests
+
+
+class TearDownBlock(Request):
+    NAME = "teardown"
+
+    def __init__(self, requests, config):
+        super().__init__(config)
+        self.requests = requests
+
+    def __repr__(self):
+        requests = [repr(req) for req in self.requests]
+        return "TearDown(requests=%s)" % requests
+
+
 class LoopBlock(Request):
     NAME = "loop"
 
@@ -239,15 +261,16 @@ class IncludeScenarioBlock(Request):
 
 
 class RequestParser(object):
-    def __init__(self, scenario, engine):
+    def __init__(self, scenario, engine, require_url):
         self.engine = engine
         self.scenario = scenario
+        self.require_url = require_url
 
-    def _parse_requests(self, raw_requests, require_url=True):
+    def _parse_requests(self, raw_requests):
         requests = []
         for key in range(len(raw_requests)):  # pylint: disable=consider-using-enumerate
             req = ensure_is_dict(raw_requests, key, "url")
-            if not require_url and "url" not in req:
+            if not self.require_url and "url" not in req:
                 req["url"] = None
             try:
                 requests.append(self._parse_request(req))
@@ -269,22 +292,20 @@ class RequestParser(object):
 
         return res
 
-    def extract_requests(self, require_url=True):
+    def extract_requests(self):
         requests = self.scenario.get("requests", [])
         requests = self._expand_transactions(requests)
-        return self._parse_requests(requests, require_url=require_url)
+        return self._parse_requests(requests)
 
 
 class HierarchicRequestParser(RequestParser):
-    def extract_requests(self, require_url=True):
+    def extract_requests(self):
         requests = self.scenario.get("requests", [])
-        return self._parse_requests(requests, require_url=require_url)
+        return self._parse_requests(requests)
 
     def _parse_request(self, req):
         if 'if' in req:
             condition = req.get("if")
-
-            # TODO: apply some checks to `condition`?
             then_clause = req.get("then", TaurusConfigError("'then' clause is mandatory for 'if' blocks"))
             then_requests = self._parse_requests(then_clause)
             else_clause = req.get("else", [])
@@ -338,6 +359,14 @@ class HierarchicRequestParser(RequestParser):
         elif 'set-variables' in req:
             mapping = req.get('set-variables')
             return SetVariables(mapping, req)
+        elif SetUpBlock.NAME in req:
+            do_block = req.get(SetUpBlock.NAME)
+            do_requests = self._parse_requests(do_block)
+            return SetUpBlock(do_requests, req)
+        elif TearDownBlock.NAME in req:
+            do_block = req.get(TearDownBlock.NAME)
+            do_requests = self._parse_requests(do_block)
+            return TearDownBlock(do_requests, req)
         elif self.scenario.get("protocol") == "mqtt":
             return MQTTRequest(req, self.scenario)
         else:
