@@ -1,15 +1,17 @@
 # Utility functions and classes for Taurus Selenium tests
 import time
-import datetime
 
-from apiritif import get_transaction_handlers, set_transaction_handlers, get_from_thread_store, get_iteration
-from apiritif import get_logging_handlers, set_logging_handlers
+from apiritif import get_transaction_handlers, set_transaction_handlers, get_from_thread_store, get_iteration, external_handler
+
 from selenium.common.exceptions import NoSuchWindowException, NoSuchFrameException, NoSuchElementException, \
     TimeoutException, UnexpectedAlertPresentException
 from selenium.webdriver.common.by import By
 from selenium.webdriver.support.wait import WebDriverWait
 from selenium.webdriver.support import expected_conditions as econd
+from selenium.webdriver.remote.webelement import WebElement
 
+from apiritif.action_plugins import BaseActionHandler
+from bzt import TaurusException
 from bzt.resources.shadow_element import ShadowElement
 
 BYS = {
@@ -31,12 +33,12 @@ def find_element_by_shadow(shadow_loc):
     css_path = [x.strip() for x in shadow_loc.split(',')]
     for p in css_path:
         if not el:
-            el = _get_driver().find_element_by_css_selector(p)
+            el = _get_driver().find_element(By.CSS_SELECTOR, p)
         else:
             curr_el = _find_element_in_shadow(el, p, False)
             if curr_el is None:
                 # try to search in the shadowRoot of the parent element
-                parent_el = el.find_element_by_xpath('..')
+                parent_el = el.find_element(By.XPATH, '..')
                 curr_el = _find_element_in_shadow(parent_el, p, True)
             el = curr_el
     return ShadowElement(el, _get_driver())
@@ -56,14 +58,26 @@ def _find_element_in_shadow(el, css_selector, raise_exception):
 def _find_by_css_selector(root, css_selector, raise_exception):
     element = None
     try:
-        element = root.find_element_by_css_selector(css_selector)
+        # Chrome 96+ returns the element as a dict that includes the id
+        if isinstance(root, dict):
+            key = list(root.keys())[0]
+            shadow_root_el = WebElement(_get_driver(), root[key])
+            element = shadow_root_el.find_element(By.CSS_SELECTOR, css_selector)
+        else:
+            element = root.find_element(By.CSS_SELECTOR, css_selector)
     except NoSuchElementException as nse:
         if raise_exception:
+            if root.tag_name == "slot":
+                raise TaurusException("Shadow DOM Slots are currently not supported in Taurus execution.")
             raise nse
         pass
     return element
 
 
+def _is_shadow_locator(locators):
+    return len(locators) == 1 and locators[0].get("shadow")
+
+
 def get_locator(locators, parent_el=None, ignore_implicit_wait=False, raise_exception=False):
     """
     :param locators: List of Dictionaries holding the locators, e.g. [{'id': 'elem_id'},
@@ -81,7 +95,7 @@ def get_locator(locators, parent_el=None, ignore_implicit_wait=False, raise_exce
     first_locator = None
     if ignore_implicit_wait:
         driver.implicitly_wait(0)
-    if len(locators) == 1 and locators[0].get("shadow"):
+    if _is_shadow_locator(locators):
         return locators
     for locator in locators:
         locator_type = list(locator.keys())[0]
@@ -154,20 +168,6 @@ def add_flow_markers():
     set_transaction_handlers(handlers)
 
 
-def add_logging_handlers(methods=None):
-    if methods is None:
-        methods = [log_into_file]
-    if methods:
-        handlers = get_logging_handlers()
-        handlers.extend(methods)
-        set_logging_handlers(handlers)
-
-
-def log_into_file(log_line):
-    with open('/tmp/apiritif_external.log', 'at') as log_file:
-        log_file.write(f"{datetime.datetime.now()} {log_line} \n")
-
-
 def _send_marker(stage, params):
     _get_driver().execute_script("/* FLOW_MARKER test-case-%s */" % stage, params)
 
@@ -175,8 +175,8 @@ def _send_marker(stage, params):
 def _send_start_flow_marker(*args, **kwargs):  # for apiritif. remove when compatibiltiy code in
     stage = "start"  # apiritif removed (http.py) and apiritif released ( > 0.9.2)
 
-    test_case, test_suite, scenario_name, data_sources = get_from_thread_store(
-        ['test_case', 'test_suite', 'scenario_name', 'data_sources']
+    test_case, test_suite, scenario_name, data_sources, action_handlers, driver = get_from_thread_store(
+        ['test_case', 'test_suite', 'scenario_name', 'data_sources', 'action_handlers', 'driver']
     )
     params = {
         "testCaseName": test_case,
@@ -185,14 +185,21 @@ def _send_start_flow_marker(*args, **kwargs):  # for apiritif. remove when compa
     if data_sources:
         params["testDataIterationId"] = get_iteration()
 
+    for handler in action_handlers:
+        handler.handle(driver.session_id, BaseActionHandler.TEST_CASE_START, params)
+
     _send_marker(stage, params)
 
 
 def _send_exit_flow_marker(*args, **kwargs):  # for apiritif. remove when compatibiltiy code in
     stage = "stop"  # apiritif removed (http.py) and apiritif released ( > 0.9.2)
     labels = "status", "message"
+    action_handlers, driver = get_from_thread_store(['action_handlers', 'driver'])
     values = get_from_thread_store(labels)
     params = dict(zip(labels, values))
+
+    for handler in action_handlers:
+        handler.handle(driver.session_id, BaseActionHandler.TEST_CASE_STOP, params)
     _send_marker(stage, params)
 
 
@@ -306,9 +313,59 @@ def dialogs_answer_on_next_confirm(value):
 
 def wait_for(condition, locators, wait_timeout=10):
     if condition.lower() in ["present", "visible", "clickable"]:
-        _wait_for_positive(condition.lower(), locators, wait_timeout)
+        if _is_shadow_locator(locators):
+            _wait_for_positive_shadow(condition.lower(), locators[0].get("shadow"), wait_timeout)
+        else:
+            _wait_for_positive(condition.lower(), locators, wait_timeout)
     elif condition.lower() in ["notpresent", "notvisible", "notclickable"]:
-        _wait_for_negative(condition.lower(), locators, wait_timeout)
+        if _is_shadow_locator(locators):
+            _wait_for_negative_shadow(condition.lower(), locators[0].get("shadow"), wait_timeout)
+        else:
+            _wait_for_negative(condition.lower(), locators, wait_timeout)
+
+
+def presence_of_shadow_element_located(locator):
+    """
+    Extends expected_conditions.py to support shadow locators
+    """
+    def _predicate(driver=None):
+        return find_element_by_shadow(locator)
+
+    return _predicate
+
+
+def _wait_for_positive_shadow(condition, shadow_loc, wait_timeout):
+    start_time = time.time()
+    while True:
+        shadow_element = None
+        try:
+            shadow_element = find_element_by_shadow(shadow_loc)
+        except NoSuchElementException:
+            pass
+        if shadow_element:
+            try:
+                element = WebDriverWait(_get_driver(), wait_timeout).until(
+                    _get_until_cond_shadow(condition, shadow_element.element, shadow_loc))
+                if element:
+                    return
+            except TimeoutException:
+                pass
+        elapsed_time = time.time() - start_time
+        if elapsed_time > wait_timeout:
+            raise NoSuchElementException("Timeout occurred while waiting for element (%s) to become '%s' " % (shadow_loc, condition))
+
+
+def _wait_for_negative_shadow(condition, shadow_loc, wait_timeout):
+    shadow_element = None
+    try:
+        shadow_element = find_element_by_shadow(shadow_loc)
+    except NoSuchElementException:
+        pass
+    if shadow_element:
+        WebDriverWait(_get_driver(), wait_timeout).until_not(
+            _get_until_cond_shadow(condition, shadow_element, shadow_loc),
+            message="Timeout occurred while waiting for element (%s) to become '%s'" %
+                    (shadow_loc, condition))
 
 
 def _wait_for_positive(condition, locators, wait_timeout):
@@ -362,6 +419,15 @@ def _get_until_cond(condition, locator):
         return econd.visibility_of_element_located(loc_tuple)
 
 
+def _get_until_cond_shadow(condition, element, shadow_locator=None):
+    if "clickable" in condition:
+        return econd.element_to_be_clickable(element)
+    if "present" in condition:
+        return presence_of_shadow_element_located(shadow_locator)
+    if "visible" in condition:
+        return econd.visibility_of(element)
+
+
 def get_loop_range(start, end, step):
     """
     :return: the range over which the loop will operate
@@ -457,3 +523,13 @@ def waiter():
                    message="Timeout occurred while waiting for page to finish loading.")
     except UnexpectedAlertPresentException:
         pass
+
+
+def action_start(action):
+    driver = _get_driver()
+    external_handler(driver.session_id if driver else None, BaseActionHandler.YAML_ACTION_START, action)
+
+
+def action_end(action):
+    driver = _get_driver()
+    external_handler(driver.session_id if driver else None, BaseActionHandler.YAML_ACTION_END, action)
